@@ -27,14 +27,14 @@ from collections import namedtuple
 
 import gi.overrides
 import gi.module
-from gi.overrides import override
+from gi.overrides import override, deprecated_attr
 from gi.repository import GLib
 from gi import PyGIDeprecationWarning
 
 from gi import _propertyhelper as propertyhelper
 from gi import _signalhelper as signalhelper
+from gi import _gi
 
-_gobject = gi._gi._gobject
 
 GObjectModule = gi.module.get_introspection_module('GObject')
 
@@ -42,7 +42,7 @@ __all__ = []
 
 
 from gi import _option as option
-sys.modules['gi._gobject.option'] = option
+option = option
 
 
 # API aliases for backwards compatibility
@@ -56,10 +56,11 @@ for name in ['markup_escape_text', 'get_application_name',
              'idle_add', 'timeout_add', 'timeout_add_seconds',
              'io_add_watch', 'child_watch_add', 'get_current_time',
              'spawn_async']:
-    globals()[name] = gi.overrides.deprecated(getattr(GLib, name), 'GLib.' + name)
+    globals()[name] = getattr(GLib, name)
+    deprecated_attr("GObject", name, "GLib." + name)
     __all__.append(name)
 
-# constants are also deprecated, but cannot mark them as such
+# deprecated constants
 for name in ['PRIORITY_DEFAULT', 'PRIORITY_DEFAULT_IDLE', 'PRIORITY_HIGH',
              'PRIORITY_HIGH_IDLE', 'PRIORITY_LOW',
              'IO_IN', 'IO_OUT', 'IO_PRI', 'IO_ERR', 'IO_HUP', 'IO_NVAL',
@@ -77,25 +78,21 @@ for name in ['PRIORITY_DEFAULT', 'PRIORITY_DEFAULT_IDLE', 'PRIORITY_HIGH',
              'OPTION_FLAG_NOALIAS', 'OPTION_ERROR_UNKNOWN_OPTION',
              'OPTION_ERROR_BAD_VALUE', 'OPTION_ERROR_FAILED', 'OPTION_REMAINING',
              'glib_version']:
-    globals()[name] = getattr(GLib, name)
+    with warnings.catch_warnings():
+        # TODO: this uses deprecated Glib attributes, silence for now
+        warnings.simplefilter('ignore', PyGIDeprecationWarning)
+        globals()[name] = getattr(GLib, name)
+    deprecated_attr("GObject", name, "GLib." + name)
     __all__.append(name)
 
 
-G_MININT8 = GLib.MININT8
-G_MAXINT8 = GLib.MAXINT8
-G_MAXUINT8 = GLib.MAXUINT8
-G_MININT16 = GLib.MININT16
-G_MAXINT16 = GLib.MAXINT16
-G_MAXUINT16 = GLib.MAXUINT16
-G_MININT32 = GLib.MININT32
-G_MAXINT32 = GLib.MAXINT32
-G_MAXUINT32 = GLib.MAXUINT32
-G_MININT64 = GLib.MININT64
-G_MAXINT64 = GLib.MAXINT64
-G_MAXUINT64 = GLib.MAXUINT64
-__all__ += ['G_MININT8', 'G_MAXINT8', 'G_MAXUINT8', 'G_MININT16',
-            'G_MAXINT16', 'G_MAXUINT16', 'G_MININT32', 'G_MAXINT32',
-            'G_MAXUINT32', 'G_MININT64', 'G_MAXINT64', 'G_MAXUINT64']
+for name in ['G_MININT8', 'G_MAXINT8', 'G_MAXUINT8', 'G_MININT16',
+             'G_MAXINT16', 'G_MAXUINT16', 'G_MININT32', 'G_MAXINT32',
+             'G_MAXUINT32', 'G_MININT64', 'G_MAXINT64', 'G_MAXUINT64']:
+    new_name = name.split("_", 1)[-1]
+    globals()[name] = getattr(GLib, new_name)
+    deprecated_attr("GObject", name, "GLib." + new_name)
+    __all__.append(name)
 
 # these are not currently exported in GLib gir, presumably because they are
 # platform dependent; so get them from our static bindings
@@ -103,7 +100,9 @@ for name in ['G_MINFLOAT', 'G_MAXFLOAT', 'G_MINDOUBLE', 'G_MAXDOUBLE',
              'G_MINSHORT', 'G_MAXSHORT', 'G_MAXUSHORT', 'G_MININT', 'G_MAXINT',
              'G_MAXUINT', 'G_MINLONG', 'G_MAXLONG', 'G_MAXULONG', 'G_MAXSIZE',
              'G_MINSSIZE', 'G_MAXSSIZE', 'G_MINOFFSET', 'G_MAXOFFSET']:
-    globals()[name] = getattr(_gobject, name)
+    new_name = name.split("_", 1)[-1]
+    globals()[name] = getattr(GLib, new_name)
+    deprecated_attr("GObject", name, "GLib." + new_name)
     __all__.append(name)
 
 
@@ -145,61 +144,67 @@ __all__ += ['TYPE_INVALID', 'TYPE_NONE', 'TYPE_INTERFACE', 'TYPE_CHAR',
 
 
 # Deprecated, use GLib directly
-Pid = GLib.Pid
-GError = GLib.GError
-OptionGroup = GLib.OptionGroup
-OptionContext = GLib.OptionContext
-__all__ += ['Pid', 'GError', 'OptionGroup', 'OptionContext']
+for name in ['Pid', 'GError', 'OptionGroup', 'OptionContext']:
+    globals()[name] = getattr(GLib, name)
+    deprecated_attr("GObject", name, "GLib." + name)
+    __all__.append(name)
 
 
 # Deprecated, use: GObject.ParamFlags.* directly
-PARAM_CONSTRUCT = GObjectModule.ParamFlags.CONSTRUCT
-PARAM_CONSTRUCT_ONLY = GObjectModule.ParamFlags.CONSTRUCT_ONLY
-PARAM_LAX_VALIDATION = GObjectModule.ParamFlags.LAX_VALIDATION
-PARAM_READABLE = GObjectModule.ParamFlags.READABLE
-PARAM_WRITABLE = GObjectModule.ParamFlags.WRITABLE
+for name in ['PARAM_CONSTRUCT', 'PARAM_CONSTRUCT_ONLY', 'PARAM_LAX_VALIDATION',
+             'PARAM_READABLE', 'PARAM_WRITABLE']:
+    new_name = name.split("_", 1)[-1]
+    globals()[name] = getattr(GObjectModule.ParamFlags, new_name)
+    deprecated_attr("GObject", name, "GObject.ParamFlags." + new_name)
+    __all__.append(name)
+
 # PARAM_READWRITE should come from the gi module but cannot due to:
 # https://bugzilla.gnome.org/show_bug.cgi?id=687615
-PARAM_READWRITE = PARAM_READABLE | PARAM_WRITABLE
-__all__ += ['PARAM_CONSTRUCT', 'PARAM_CONSTRUCT_ONLY', 'PARAM_LAX_VALIDATION',
-            'PARAM_READABLE', 'PARAM_WRITABLE', 'PARAM_READWRITE']
+PARAM_READWRITE = GObjectModule.ParamFlags.READABLE | \
+    GObjectModule.ParamFlags.WRITABLE
+__all__.append("PARAM_READWRITE")
+
+# READWRITE is part of ParamFlags since glib 2.42. Only mark PARAM_READWRITE as
+# deprecated in case ParamFlags.READWRITE is available. Also include the glib
+# version in the warning so it's clear that this needs a newer glib, unlike
+# the other ParamFlags related deprecations.
+# https://bugzilla.gnome.org/show_bug.cgi?id=726037
+if hasattr(GObjectModule.ParamFlags, "READWRITE"):
+    deprecated_attr("GObject", "PARAM_READWRITE",
+                    "GObject.ParamFlags.READWRITE (glib 2.42+)")
 
 
 # Deprecated, use: GObject.SignalFlags.* directly
-SIGNAL_ACTION = GObjectModule.SignalFlags.ACTION
-SIGNAL_DETAILED = GObjectModule.SignalFlags.DETAILED
-SIGNAL_NO_HOOKS = GObjectModule.SignalFlags.NO_HOOKS
-SIGNAL_NO_RECURSE = GObjectModule.SignalFlags.NO_RECURSE
-SIGNAL_RUN_CLEANUP = GObjectModule.SignalFlags.RUN_CLEANUP
-SIGNAL_RUN_FIRST = GObjectModule.SignalFlags.RUN_FIRST
-SIGNAL_RUN_LAST = GObjectModule.SignalFlags.RUN_LAST
-__all__ += ['SIGNAL_ACTION', 'SIGNAL_DETAILED', 'SIGNAL_NO_HOOKS',
-            'SIGNAL_NO_RECURSE', 'SIGNAL_RUN_CLEANUP', 'SIGNAL_RUN_FIRST',
-            'SIGNAL_RUN_LAST']
-
+for name in ['SIGNAL_ACTION', 'SIGNAL_DETAILED', 'SIGNAL_NO_HOOKS',
+             'SIGNAL_NO_RECURSE', 'SIGNAL_RUN_CLEANUP', 'SIGNAL_RUN_FIRST',
+             'SIGNAL_RUN_LAST']:
+    new_name = name.split("_", 1)[-1]
+    globals()[name] = getattr(GObjectModule.SignalFlags, new_name)
+    deprecated_attr("GObject", name, "GObject.SignalFlags." + new_name)
+    __all__.append(name)
 
 # Static types
-GBoxed = _gobject.GBoxed
-GEnum = _gobject.GEnum
-GFlags = _gobject.GFlags
-GInterface = _gobject.GInterface
-GObject = _gobject.GObject
-GObjectWeakRef = _gobject.GObjectWeakRef
-GParamSpec = _gobject.GParamSpec
-GPointer = _gobject.GPointer
-GType = _gobject.GType
-Warning = _gobject.Warning
+GBoxed = _gi.GBoxed
+GEnum = _gi.GEnum
+GFlags = _gi.GFlags
+GInterface = _gi.GInterface
+GObject = _gi.GObject
+GObjectWeakRef = _gi.GObjectWeakRef
+GParamSpec = _gi.GParamSpec
+GPointer = _gi.GPointer
+GType = _gi.GType
+Warning = _gi.Warning
 __all__ += ['GBoxed', 'GEnum', 'GFlags', 'GInterface', 'GObject',
             'GObjectWeakRef', 'GParamSpec', 'GPointer', 'GType',
             'Warning']
 
 
-features = _gobject.features
-list_properties = _gobject.list_properties
-new = _gobject.new
-pygobject_version = _gobject.pygobject_version
+features = _gi.features
+list_properties = _gi.list_properties
+new = _gi.new
+pygobject_version = _gi.pygobject_version
 threads_init = GLib.threads_init
-type_register = _gobject.type_register
+type_register = _gi.type_register
 __all__ += ['features', 'list_properties', 'new',
             'pygobject_version', 'threads_init', 'type_register']
 
@@ -216,19 +221,22 @@ class Value(GObjectModule.Value):
         if self._free_on_dealloc and self.g_type != TYPE_INVALID:
             self.unset()
 
+        # We must call base class __del__() after unset.
+        super(Value, self).__del__()
+
     def set_boxed(self, boxed):
         # Workaround the introspection marshalers inability to know
         # these methods should be marshaling boxed types. This is because
         # the type information is stored on the GValue.
-        _gobject._gvalue_set(self, boxed)
+        _gi._gvalue_set(self, boxed)
 
     def get_boxed(self):
-        return _gobject._gvalue_get(self)
+        return _gi._gvalue_get(self)
 
     def set_value(self, py_value):
         gtype = self.g_type
 
-        if gtype == _gobject.TYPE_INVALID:
+        if gtype == _gi.TYPE_INVALID:
             raise TypeError("GObject.Value needs to be initialized first")
         elif gtype == TYPE_BOOLEAN:
             self.set_boolean(py_value)
@@ -343,6 +351,7 @@ class Value(GObjectModule.Value):
     def __repr__(self):
         return '<Value (%s) %s>' % (self.g_type.name, self.get_value())
 
+
 Value = override(Value)
 __all__.append('Value')
 
@@ -353,6 +362,7 @@ def type_from_name(name):
         raise RuntimeError('unknown type name: %s' % name)
     return type_
 
+
 __all__.append('type_from_name')
 
 
@@ -362,6 +372,7 @@ def type_parent(type_):
         raise RuntimeError('no parent for type')
     return parent
 
+
 __all__.append('type_parent')
 
 
@@ -376,6 +387,7 @@ def signal_list_ids(type_):
     _validate_type_for_signal_method(type_)
     return GObjectModule.signal_list_ids(type_)
 
+
 __all__.append('signal_list_ids')
 
 
@@ -383,6 +395,7 @@ def signal_list_names(type_):
     ids = signal_list_ids(type_)
     return tuple(GObjectModule.signal_name(i) for i in ids)
 
+
 __all__.append('signal_list_names')
 
 
@@ -390,34 +403,39 @@ def signal_lookup(name, type_):
     _validate_type_for_signal_method(type_)
     return GObjectModule.signal_lookup(name, type_)
 
+
 __all__.append('signal_lookup')
 
 
-def signal_query(id_or_name, type_=None):
-    SignalQuery = namedtuple('SignalQuery',
-                             ['signal_id',
-                              'signal_name',
-                              'itype',
-                              'signal_flags',
-                              'return_type',
-                              # n_params',
-                              'param_types'])
+SignalQuery = namedtuple('SignalQuery',
+                         ['signal_id',
+                          'signal_name',
+                          'itype',
+                          'signal_flags',
+                          'return_type',
+                          # n_params',
+                          'param_types'])
 
-    # signal_query needs to use a static method until the following bugs are fixed:
-    # https://bugzilla.gnome.org/show_bug.cgi?id=687550
-    # https://bugzilla.gnome.org/show_bug.cgi?id=687545
-    # https://bugzilla.gnome.org/show_bug.cgi?id=687541
+
+def signal_query(id_or_name, type_=None):
     if type_ is not None:
         id_or_name = signal_lookup(id_or_name, type_)
 
-    res = _gobject.signal_query(id_or_name)
+    res = GObjectModule.signal_query(id_or_name)
     if res is None:
         return None
 
-    # Return a named tuple which allows indexing like the static bindings
-    # along with field like access of the gi struct.
-    # Note however that the n_params was not returned from the static bindings.
-    return SignalQuery(*res)
+    if res.signal_id == 0:
+        return None
+
+    # Return a named tuple to allows indexing which is compatible with the
+    # static bindings along with field like access of the gi struct.
+    # Note however that the n_params was not returned from the static bindings
+    # so we must skip over it.
+    return SignalQuery(res.signal_id, res.signal_name, res.itype,
+                       res.signal_flags, res.return_type,
+                       tuple(res.param_types))
+
 
 __all__.append('signal_query')
 
@@ -454,6 +472,7 @@ def signal_handler_block(obj, handler_id):
     GObjectModule.signal_handler_block(obj, handler_id)
     return _HandlerBlockManager(obj, handler_id)
 
+
 __all__.append('signal_handler_block')
 
 
@@ -475,6 +494,7 @@ def signal_parse_name(detailed_signal, itype, force_detail_quark):
     else:
         raise ValueError('%s: unknown signal name: %s' % (itype, detailed_signal))
 
+
 __all__.append('signal_parse_name')
 
 
@@ -482,6 +502,7 @@ def remove_emission_hook(obj, detailed_signal, hook_id):
     signal_id, detail = signal_parse_name(detailed_signal, obj, True)
     GObjectModule.signal_remove_emission_hook(signal_id, hook_id)
 
+
 __all__.append('remove_emission_hook')
 
 
@@ -492,6 +513,7 @@ def signal_accumulator_first_wins(ihint, return_accu, handler_return, user_data=
     # Stop emission but return the result of the last handler
     return (False, handler_return)
 
+
 __all__.append('signal_accumulator_first_wins')
 
 
@@ -499,13 +521,14 @@ def signal_accumulator_true_handled(ihint, return_accu, handler_return, user_dat
     # Stop emission if the last handler returns True
     return (not handler_return, handler_return)
 
+
 __all__.append('signal_accumulator_true_handled')
 
 
 # Statically bound signal functions which need to clobber GI (for now)
 
-add_emission_hook = _gobject.add_emission_hook
-signal_new = _gobject.signal_new
+add_emission_hook = _gi.add_emission_hook
+signal_new = _gi.signal_new
 
 __all__ += ['add_emission_hook', 'signal_new']
 
@@ -573,23 +596,23 @@ class Object(GObjectModule.Object):
 
     # The following methods are static APIs which need to leap frog the
     # gi methods until we verify the gi methods can replace them.
-    get_property = _gobject.GObject.get_property
-    get_properties = _gobject.GObject.get_properties
-    set_property = _gobject.GObject.set_property
-    set_properties = _gobject.GObject.set_properties
-    bind_property = _gobject.GObject.bind_property
-    connect = _gobject.GObject.connect
-    connect_after = _gobject.GObject.connect_after
-    connect_object = _gobject.GObject.connect_object
-    connect_object_after = _gobject.GObject.connect_object_after
-    disconnect_by_func = _gobject.GObject.disconnect_by_func
-    handler_block_by_func = _gobject.GObject.handler_block_by_func
-    handler_unblock_by_func = _gobject.GObject.handler_unblock_by_func
-    emit = _gobject.GObject.emit
-    chain = _gobject.GObject.chain
-    weak_ref = _gobject.GObject.weak_ref
-    __copy__ = _gobject.GObject.__copy__
-    __deepcopy__ = _gobject.GObject.__deepcopy__
+    get_property = _gi.GObject.get_property
+    get_properties = _gi.GObject.get_properties
+    set_property = _gi.GObject.set_property
+    set_properties = _gi.GObject.set_properties
+    bind_property = _gi.GObject.bind_property
+    connect = _gi.GObject.connect
+    connect_after = _gi.GObject.connect_after
+    connect_object = _gi.GObject.connect_object
+    connect_object_after = _gi.GObject.connect_object_after
+    disconnect_by_func = _gi.GObject.disconnect_by_func
+    handler_block_by_func = _gi.GObject.handler_block_by_func
+    handler_unblock_by_func = _gi.GObject.handler_unblock_by_func
+    emit = _gi.GObject.emit
+    chain = _gi.GObject.chain
+    weak_ref = _gi.GObject.weak_ref
+    __copy__ = _gi.GObject.__copy__
+    __deepcopy__ = _gi.GObject.__deepcopy__
 
     def freeze_notify(self):
         """Freezes the object's property-changed notification queue.
@@ -625,9 +648,9 @@ class Object(GObjectModule.Object):
         """
         flags = kwargs.get('connect_flags', 0)
         if flags & GObjectModule.ConnectFlags.AFTER:
-            connect_func = _gobject.GObject.connect_after
+            connect_func = _gi.GObject.connect_after
         else:
-            connect_func = _gobject.GObject.connect
+            connect_func = _gi.GObject.connect
 
         if flags & GObjectModule.ConnectFlags.SWAPPED:
             if len(data) != 1:
@@ -699,4 +722,5 @@ SignalOverride = signalhelper.SignalOverride
 # Deprecated naming "property" available for backwards compatibility.
 # Keep this at the end of the file to avoid clobbering the builtin.
 property = Property
+deprecated_attr("GObject", "property", "GObject.Property")
 __all__ += ['Property', 'Signal', 'SignalOverride', 'property']
