@@ -45,7 +45,7 @@ pyg_source_prepare(GSource *source, gint *timeout)
     gboolean got_err = TRUE;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
     t = PyObject_CallMethod(pysource->obj, "prepare", NULL);
 
@@ -81,7 +81,7 @@ bail:
 
     Py_XDECREF(t);
 
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 
     return ret;
 }
@@ -94,7 +94,7 @@ pyg_source_check(GSource *source)
     gboolean ret;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
     t = PyObject_CallMethod(pysource->obj, "check", NULL);
 
@@ -106,7 +106,7 @@ pyg_source_check(GSource *source)
 	Py_DECREF(t);
     }
 
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 
     return ret;
 }
@@ -119,7 +119,7 @@ pyg_source_dispatch(GSource *source, GSourceFunc callback, gpointer user_data)
     gboolean ret;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
     if (callback) {
 	tuple = user_data;
@@ -141,7 +141,7 @@ pyg_source_dispatch(GSource *source, GSourceFunc callback, gpointer user_data)
 	Py_DECREF(t);
     }
 
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 
     return ret;
 }
@@ -153,7 +153,7 @@ pyg_source_finalize(GSource *source)
     PyObject *func, *t;
     PyGILState_STATE state;
 
-    state = pyglib_gil_state_ensure();
+    state = PyGILState_Ensure();
 
     func = PyObject_GetAttrString(pysource->obj, "finalize");
     if (func) {
@@ -165,9 +165,11 @@ pyg_source_finalize(GSource *source)
 	} else {
 	    Py_DECREF(t);
 	}
+    } else {
+        PyErr_Clear ();
     }
 
-    pyglib_gil_state_release(state);
+    PyGILState_Release(state);
 }
 
 static GSourceFuncs pyg_source_funcs =
@@ -239,10 +241,8 @@ pyg_source_new (void)
     source = (PyGRealSource*) g_source_new (&pyg_source_funcs, sizeof (PyGRealSource));
 
     py_type = _pygi_type_import_by_name ("GLib", "Source");
-    /* Full ownership transfer of the source, this will be free'd with g_boxed_free. */
-    source->obj = _pygi_boxed_new ( (PyTypeObject *) py_type, source,
-                                    FALSE, /* copy_boxed */
-                                    0);    /* slice_allocated */
+    /* g_source_new uses malloc, not slices */
+    source->obj = _pygi_boxed_new ( (PyTypeObject *) py_type, source, FALSE, 0);
 
     return source->obj;
 }
