@@ -2,13 +2,15 @@
 # coding: UTF-8
 # vim: tabstop=4 shiftwidth=4 expandtab
 
+from __future__ import absolute_import
+
 import contextlib
 import unittest
 import time
 import sys
 import warnings
 
-from helper import ignore_gi_deprecation_warnings, capture_glib_warnings
+from .helper import ignore_gi_deprecation_warnings, capture_glib_warnings
 
 import gi.overrides
 import gi.types
@@ -1391,6 +1393,10 @@ class TestTreeModel(unittest.TestCase):
         self.assertEqual(p1[2], 3)
         self.assertRaises(IndexError, p1.__getitem__, 3)
 
+    def test_tree_path_empty(self):
+        p1 = Gtk.TreePath.new()
+        assert str(p1) == ""
+
     def test_tree_model(self):
         tree_store = Gtk.TreeStore(int, str)
 
