@@ -1,10 +1,13 @@
 # -*- Mode: Python -*-
 
+from __future__ import absolute_import
+
 import unittest
-import testhelper
 
 from gi.repository import GLib
 
+from . import testhelper
+
 
 class TestThread(unittest.TestCase):
     def setUp(self):
